<?php

/**
 * Course object for claiming course credit.
 */
class CourseObjectCredit extends CourseObject {

  function optionsDefinition() {
    $defaults = parent::optionsDefinition();
    $defaults['required'] = 0;
    $defaults['credit_types'] = [];
    return $defaults;
  }

  function take() {
    global $user;

    $config_types = $this->getOption('credit_types');
    $awarded = course_credit_get_credit_awarded($this->getCourseNid(), $user->uid);

    $show_claim = 1;
    if (array_filter($config_types)) {
      $enabled_types = array_filter($config_types);
      if (array_intersect_key($enabled_types, $awarded)) {
        // User claimed a filtered type already.
        $show_claim = 0;
      }
    }
    elseif ($awarded) {
      $show_claim = 0;
    }

    if ($show_claim) {
      // No credit awarded yet, let's try to award credit.

      if (course_credit_must_claim($this->getCourse()->getNode(), $user)) {
        // User must claim variable credit, so show them the claiming form.
        $form = drupal_get_form('course_credit_app_form', $this->getCourse()->getNode());

        if (array_filter($config_types)) {
          // Credit types were selected to be filtered.
          foreach (element_children($form['credits_claimed']) as $type) {
            if (empty($config_types[$type])) {
              unset($form['credits_claimed'][$type]);
            }
          }
        }

        if (element_children($form['credits_claimed'])) {
          return $form;
        }
      }

      // There wasn't any variable credit to be awarded so let's try to claim
      // nonvariable credit.
      course_credit_award_credit($this->getCourse()->getNode(), $user);
    }

    // Get credit again to see if anything changed.
    $awarded = course_credit_get_credit_awarded($this->getCourseNid(), $user->uid);
    if ($awarded) {
      return theme('course_credit_awarded', array('course_credit_awarded' => $awarded));
    }
    else {
      return t('Sorry, there is no credit to be awarded.');
    }
  }

  function save() {
    $this->setInstanceId($this->getCourseNid());
    return parent::save();
  }

  function getCloneAbility() {
    return TRUE;
  }

  function optionsForm(&$form, &$form_state) {
    parent::optionsForm($form, $form_state);

    $config = $this->getOptions();

    // Get active instances on the course.
    $types = [];
    foreach ($this->getCourse()->getNode()->course_credit as $credit_type => $instance) {
      if ($instance->active) {
        $types[$credit_type] = $instance->title;
      }
    }

    $form['credit_types'] = [
      '#title' => t('Credit types'),
      '#description' => t('If the credit types selected below are variable, they will be available for claiming on this course object. If no credit types are selected here, all will be available. Regardless of what is selected here, users must be eligible for the credit type in order for it to be awarded. <strong>Non-variable credit will always be awarded and ignore any selections below.</strong>'),
      '#type' => 'checkboxes',
      '#default_value' => $config['credit_types'],
      '#options' => $types,
      '#access' => !empty($types),
    ];

    return $form;
  }

  /**
   * Do not allow credit types to be shared across objects.
   */
  public function optionsValidate(&$form, &$form_state) {
    parent::optionsValidate($form, $form_state);

    foreach ($this->getCourse()->getObjects() as $courseObject) {
      if ($courseObject->identifier() != $this->identifier() && $courseObject->getComponent() == 'credit') {
        $types = $courseObject->getOption('credit_types');
        if ($match = array_intersect_key(array_filter($types), array_filter($form_state['values']['credit_types']))) {
          $types = $form['credit']['credit_types']['#options'];
          form_error($form['credit'], t('The credit type !type is already configured on another object.', ['!type' => $types[current($match)]]));
        }
      }
    }
  }

}
